open Lexing
open Format
open Options

module Time = struct

  open Unix
    
  let u = ref 0.0
    
  let start () = u:=(times()).tms_utime

  let get () = (times()).tms_utime -. !u

end

module Sat = Sat

type result = 
  | Valid
  | Invalid
  | DontKnow

let process_decl env = function
  | Why_typing.Assume(f,loc) -> Sat.assume env (f,0,None)
  | Why_typing.Query(n,f,loc) -> 
      let f = if !satmode then f else Formula.mk_not f in
	Sat.unsat env (f,0,None) stopb; 
	ignore (if !satmode then raise Sat.Unsat else raise Sat.Sat);
	env

let make_ast s = 
  let ast = Why_parser.file Why_lexer.token (Lexing.from_string s) in
  Why_typing.file ast

let remake_trs_ast s=
  let ast_trs = Why_parser.triggers_string Why_lexer.token (Lexing.from_string s) in
    ast_trs
      
let run a =
  try 
    let q = Why_typing.make_cnf a in
    ignore (Queue.fold process_decl Sat.empty q);
    assert false
  with 
    | Sat.Sat -> if !satmode then Valid else Invalid
    | Sat.Unsat -> if !satmode then Invalid else Valid
    | Sat.I_dont_know -> DontKnow
