type 'a mine = Yes | No of 'a

module type RELATION = sig
  type t
  type elt

  exception Inconsistent    
  val empty : t
  val assume : 
    t -> Literal.Set.t -> (Term.t -> elt) -> (Term.t -> Term.t list) -> 
    t * Literal.SetEq.t
  val query : 
    Literal.t -> (Term.t -> elt) -> (Term.t -> Term.t list) -> t -> bool 
end


module type THEORY = sig

  type t

  module Map : Map.S with type key = t
  module Set : Set.S with type elt = t

  exception Unsolvable
  val name : string
  val is_mine_t : Term.t -> bool
  val is_mine_r : t -> Term.t mine
  val is_mine_a : Literal.t -> bool
  val normal_form : Literal.t -> Literal.t

  val make : Term.t -> t
  val leaves : t -> Term.Set.t
  val subst : Term.t -> t -> t -> t

  val compare : t -> t -> int

  val solve : t -> t ->  (Term.t * t) list

  val print : Format.formatter -> t -> unit

  module R : RELATION with type elt = t
end


