open Hashcons

type binop = Plus | Minus | Mult | Div | Modulo
type t = 
    Name of Hstring.t
  | Int of Hstring.t
  | Binop of binop 
  | Var of Hstring.t

let unit = Name (Hstring.make "()")

let name s = Name (Hstring.make s)
let var s = Var (Hstring.make s)
let int i = Int (Hstring.make i)
let real r = Name (Hstring.make r)

let plus = Binop Plus
let minus = Binop Minus
let mult = Binop Mult
let div = Binop Div
let modulo = Binop Modulo

let compare s1 s2 =  match s1,s2 with
    Name n1 , Name n2 -> Hstring.compare n1 n2
  | Name _ , _ ->  -1
  | _ , Name _ -> 1
  | Var n1 , Var n2 -> Hstring.compare n1 n2
  | Var _ , _ -> -1
  | _ , Var _ -> 1
  | Int i1 , Int i2 -> Hstring.compare i1 i2
  | Int _ , _ -> -1
  | _ , Int _ -> 1
  | _  -> Pervasives.compare s1 s2
  
let equal s1 s2 = compare s1 s2 = 0

let hash = function
    Name n -> Hstring.hash n * 19
  | Var n (*| Int n*) -> Hstring.hash n * 19 + 1
  | s -> Hashtbl.hash s
	
let to_string =  function
    Name n -> Hstring.view n
  | Var x -> "["^(Hstring.view x)^"]"
  | Int n -> Hstring.view n
  | Binop Plus -> "+" 
  | Binop Minus -> "-" 
  | Binop Mult -> "*"
  | Binop Div -> "*"
  | Binop Modulo -> "%"

let print fmt s = Format.fprintf fmt "%s" (to_string s)
	
  
(*let is_const s = match view s with Const _ -> true | _ -> false

let value s = 
  match view s with Const n -> n 
    | _ -> raise Not_found
*)

let is_c s = false (*s.symb = Name("f")*)
let is_ac s = false (*s.symb = Name("h")*)

let dummy = Name (Hstring.make "_one")

let fresh = 
  let cpt = ref 0 in
  fun () -> incr cpt; name ("__sko"^(string_of_int !cpt))

module Map =
  Map.Make(struct type t' = t type t=t' let compare=compare end)

module Set = 
  Set.Make(struct type t' = t type t=t' let compare=compare end)

